@@ -1,9 +1,8 @@
-#$Id: Agent.pm 427 2014-05-13 19:47:26Z maj $
 use v5.10;
 package REST::Neo4p::Agent;
 use REST::Neo4p::Exceptions;
-use File::Temp qw(tempfile);
 use JSON;
+use File::Temp;
 use Carp qw(croak carp);
 use strict;
 use warnings;
@@ -11,7 +10,7 @@ use warnings;
 our @ISA;
 our $VERSION;
 BEGIN {
-  $REST::Neo4p::Agent::VERSION = '0.2251';
+  $REST::Neo4p::Agent::VERSION = '0.4000';
 }
 
 our $AUTOLOAD;
@@ -22,7 +21,7 @@ our $RETRY_WAIT = 5;
 sub new {
   my $class = shift;
   my %args = @_;
-  my $mod = delete $args{agent_module};
+  my $mod = delete $args{agent_module} || 'LWP::UserAgent';
   die "No user agent module specified" unless $mod;
   $mod = join('::','REST::Neo4p::Agent',$mod);
   eval "require $mod;1" or REST::Neo4p::LocalException->throw("Module $mod is not available\n");
@@ -64,8 +63,9 @@ sub connect {
    }
   my $resp = $self->get($self->server_url);
   unless ($resp->is_success) {
-    REST::Neo4p::CommException->throw( code => $resp->code,
-				       message => $resp->message );
+    my $exc = $resp->code == 401 ? 'AuthException' : 'CommException';
+      "REST::Neo4p::$exc"->throw( code => $resp->code,
+					 message => $resp->message );
   }
   my $json =  $JSON->decode($resp->content);
   # add the discovered URLs to the object hash, keyed by 
@@ -78,7 +78,8 @@ sub connect {
   }
   $resp = $self->get($self->{_actions}{data});
   unless ($resp->is_success) {
-    REST::Neo4p::CommException->throw( code => $resp->code,
+    my $exc = $resp->code == 401 ? 'AuthException' : 'CommException';
+    "REST::Neo4p::$exc"->throw( code => $resp->code,
 				       message => $resp->message." (connect phase 2)\n" );
   }
   $json = $JSON->decode($resp->content);
@@ -98,7 +99,6 @@ sub connect {
 # _add_to_batch_queue
 # takes a request and converts to a Neo4j REST batch-friendly
 # hash
-# VERY internal and experimental
 # $url : rest endpoint that would be called ordinarily
 # $rq : [get|delete|post|put]
 # $content : hashref of rq content (post and put)
@@ -127,7 +127,6 @@ sub execute_batch {
     REST::Neo4p::LocalException->throw("Agent not in batch mode; can't execute batch\n");
   }
   return unless ($self->batch_length);
-#  my ($tfh, $tfname) = tempfile;
   my $tfh = File::Temp->new;
   $self->batch_mode(0);
   my @chunk;
@@ -160,11 +159,17 @@ sub no_stream { shift->remove_header('X-Stream') }
 sub stream { shift->add_header('X-Stream' => 'true') }
 
 # autoload getters for discovered neo4j rest urls
+# when the agent module is Neo4j::Driver, all actions are explicitly defined,
+# so any call falling through to AUTOLOAD is an error
 
 sub AUTOLOAD {
   my $self = shift;
   my $method = $AUTOLOAD;
   $method =~ s/.*:://;
+  if ($self->isa('REST::Neo4p::Agent::Neo4j::Driver')) {
+    # an error
+    REST::Neo4p::LocalException->throw( "REST::Neo4p::Agent::Neo4j::Driver does not define method '$method'\n" );
+  }
   my ($rq, $action) = $method =~ /^(get_|post_|put_|delete_)*(.*)$/;
   unless (grep /^$action$/,keys %{$self->{_actions}}) {
     REST::Neo4p::LocalException->throw( __PACKAGE__." does not define method '$method'\n" );
@@ -185,7 +190,7 @@ sub AUTOLOAD {
       }
     }
     elsif ($e = Exception::Class->caught()) {
-      ref $e ? $e->rethrow : die $e;
+      (ref $e && $e->can("rethrow")) ? $e->rethrow : die $e;
     }
     else {
       last; # success
@@ -221,13 +226,14 @@ sub __do_request {
       }
       $url.='?'.join('&',@params) if @params;
       if ($self->batch_mode) {
-	$url = ($url_components[0] =~ /{[0-9]+}/) ? $url_components[0] : $url; # index batch object kludge
+	$url = (@url_components && ($url_components[0] =~ /{[0-9]+}/)) ? $url_components[0] : $url; # index batch object kludge
 
 	@_ = ($self, 
 	      $url,
 	      $rq);
 	goto &_add_to_batch_queue; # short circuit to _add_to_batch_queue
       }
+# request made here:
       $resp = $self->{_raw_response} = $self->$rq($url);
     }
     when (/post|put/) {
@@ -246,6 +252,7 @@ sub __do_request {
 	goto &_add_to_batch_queue;
       }
       $content = $JSON->encode($content) if $content && !$self->isa('Mojo::UserAgent');
+# request made here
       $resp  = $self->{_raw_response} = $self->$rq($url, 'Content-Type' => 'application/json', Content=> $content, %$addl_headers);
       1;
     }
@@ -295,6 +302,21 @@ sub __do_request {
   $self->{_location} = $resp->header('Location');
 }
 
+sub neo4j_version {
+  my $self = shift;
+  my $v = my $a = $self->{_actions}{neo4j_version};
+  return unless defined $v;
+  my ($major, $minor, $patch, $milestone) =
+    $a =~ /^(?:([0-9]+)\.)(?:([0-9]+)\.)?([0-9]+)?(?:-M([0-9]+))?/;
+  wantarray ? ($major,$minor,$patch,$milestone) : $v;
+}
+
+sub is_version_4 {
+  my ($maj,@rest) = shift->neo4j_version;
+  return $maj >= 4;
+}
+
+
 sub DESTROY {}
 
 =head1 NAME
@@ -358,13 +380,20 @@ is a default header for requests. The server responds to requests with
 chunked content, which is handled correctly by any of the underlying
 user agents.
 
-Streaming can be stopped by calling
+L<REST::Neo4p::Query> and L<REST::Neo4p::Batch> take advantage of
+streamed responsed by retrieving and returning JSON objects
+incrementally and (with the L<Mojo::UserAgent> backend) in a
+non-blocking way. New Neo4j server versions may break the incremental
+parsing. If this happens,  L<make a
+ticket|https://rt.cpan.org/Public/Bug/Report.html?Queue=REST-Neo4p>. In
+the meantime, you should be able to keep things going (albeit more
+slowly) by turning off streaming at the agent:
 
- $agent->no_stream
+ REST::Neo4p->agent->no_stream;
 
-and started with
+Streaming responses can be requested again by issuing
 
- $agent->stream
+ REST::Neo4p->agent->stream
 
 For batch API features, see L</Batch Mode>.
 
@@ -413,7 +442,7 @@ Returns the Neo4j server admin url.
 
 =item relationship_index()
 
-=item extensions_info
+=item extensions_info()
 
 =item relationship_types()
 
@@ -435,8 +464,9 @@ methods to make requests directly.
 =item neo4j_version()
 
  $version = $agent->neo4j_version;
+ ($major, $minor, $patch, $milestone) = $agent->neo4j_version;
 
-Returns the version string of the connected Neo4j server.
+Returns the version string/components of the connected Neo4j server.
 
 =item available_actions()
 
@@ -476,7 +506,7 @@ present, is a hashref containing additional request headers.
  # create a new node with given properties
  $agent->post_node({ name => 'Wanda' });
  # do a cypher query and save content to file
- $agent->post_cypher([], { query => 'START n=node(*) RETURN n', params=>{}},
+ $agent->post_cypher([], { query => 'MATCH (n) RETURN n', params=>{}},
                      { ':content_file' => $my_file_name });
 
 Makes a POST request to the REST endpoint mapped to {action}. The first
@@ -525,6 +555,10 @@ Adds C<X-Stream: true> to the default headers.
 
 =head1 Batch Mode
 
+B<Neo4j version 4.0+>: I<Batch mode is a Neo4j REST API feature that bit
+the big one along with that API. The Neo4j::Driver agent will complain
+if you use these methods.>
+
 When the agent is in batch mode, the usual request calls are not
 executed immediately, but added to a queue. The L</execute_batch()>
 method sends the queued calls in the format required by the Neo4p REST
@@ -548,7 +582,7 @@ Set/get current agent mode.
  }
 
 Returns current queue length. Throws
-L<REST::Neo4p::Exceptions|REST::Neo4p::LocalException> if agent not in
+L<REST::Neo4p::LocalException|REST::Neo4p::Exceptions> if agent not in
 batch mode.
 
 =item execute_batch()
@@ -562,7 +596,7 @@ batch mode.
 
 Processes the queued calls and returns the decoded json response from
 server in a temporary file. Returns with undef if batch length is zero.
-Throws L<REST::Neo4p::Exceptions|REST::Neo4p::LocalException> if not in batch mode.
+Throws L<REST::Neo4p::LocalException|REST::Neo4p::Exceptions> if not in batch mode.
 
 Second form takes an integer argument; this will submit the next [integer]
 jobs and return the server response in the tempfile. The batch length is
@@ -591,7 +625,7 @@ has default value of 1024.
 
 =head1 LICENSE
 
-Copyright (c) 2012-2014 Mark A. Jensen. This program is free software; you
+Copyright (c) 2012-2020 Mark A. Jensen. This program is free software; you
 can redistribute it and/or modify it under the same terms as Perl
 itself.
 
