@@ -1,7 +1,10 @@
-#$Id: Constrain.pm 427 2014-05-13 19:47:26Z maj $
+#$Id$
 package REST::Neo4p::Constrain;
 use base 'Exporter';
 use REST::Neo4p::Constraint qw(:all);
+use REST::Neo4p::Constraint::Property;
+use REST::Neo4p::Constraint::Relationship;
+use REST::Neo4p::Constraint::RelationshipType;
 use REST::Neo4p::Exceptions;
 use strict;
 use warnings;
@@ -9,7 +12,7 @@ no warnings qw(once redefine);
 
 
 BEGIN {
-  $REST::Neo4p::Constrain::VERSION = '0.2251';
+  $REST::Neo4p::Constrain::VERSION = '0.4000';
 }
 our @EXPORT = qw(create_constraint drop_constraint constrain relax);
 our @VALIDATE = qw(validate_properties validate_relationship validate_relationship_type);
@@ -75,7 +78,7 @@ sub create_constraint {
 	REST::Neo4p::ConstraintSpecException->throw($e->message);
       }
       if ($e = Exception::Class->caught()) {
-	ref $e ? $e->rethrow : die $e;
+	(ref $e && $e->can("rethrow")) ? $e->rethrow : die $e;
       }
       last;
     };
@@ -96,7 +99,7 @@ sub create_constraint {
 	REST::Neo4p::ConstraintSpecException->throw($e->message);
       }
       if ($e = Exception::Class->caught()) {
-	ref $e ? $e->rethrow : die $e;
+	(ref $e && $e->can("rethrow")) ? $e->rethrow : die $e;
       }
       last;
     };
@@ -119,7 +122,7 @@ sub create_constraint {
 	REST::Neo4p::ConstraintSpecException->throw($e->message);
       }
       if ($e = Exception::Class->caught()) {
-	ref $e ? $e->rethrow : die $e;
+	(ref $e && $e->can("rethrow")) ? $e->rethrow : die $e;
       }
       last;
     };
@@ -141,7 +144,7 @@ sub create_constraint {
 	REST::Neo4p::ConstraintSpecException->throw($e->message);
       }
       if ($e = Exception::Class->caught()) {
-	ref $e ? $e->rethrow : die $e;
+	(ref $e && $e->can("rethrow")) ? $e->rethrow : die $e;
       }
       last;
     };
@@ -792,7 +795,7 @@ L<REST::Neo4p>, L<REST::Neo4p::Constraint>
 
 =head1 LICENSE
 
-Copyright (c) 2012-2014 Mark A. Jensen. This program is free software; you
+Copyright (c) 2012-2020 Mark A. Jensen. This program is free software; you
 can redistribute it and/or modify it under the same terms as Perl
 itself.
 
