@@ -1,11 +1,8 @@
-#$Id: Constraint.pm 427 2014-05-13 19:47:26Z maj $
+#$Id$
 package REST::Neo4p::Constraint;
 use base 'Exporter';
 use REST::Neo4p;
 use REST::Neo4p::Exceptions;
-use REST::Neo4p::Constraint::Property;
-use REST::Neo4p::Constraint::Relationship;
-use REST::Neo4p::Constraint::RelationshipType;
 use JSON;
 use Data::Dumper;
 
@@ -22,7 +19,7 @@ our %EXPORT_TAGS = (
   all => [@EXPORT,@EXPORT_OK]
 );
 
-our $jobj = JSON->new();
+our $jobj = JSON->new->utf8;
 $jobj->allow_blessed(1);
 $jobj->convert_blessed(1);
 my $regex_to_json = sub {
@@ -34,7 +31,7 @@ my $regex_to_json = sub {
 };
 
 BEGIN {
-  $REST::Neo4p::Constraint::VERSION = '0.2251';
+  $REST::Neo4p::Constraint::VERSION = '0.4000';
 }
 
 # valid constraint types
@@ -498,7 +495,7 @@ L<REST::Neo4p::Constraint::RelationshipType>. L<REST::Neo4p::Node>, L<REST::Neo4
 
 =head1 LICENSE
 
-Copyright (c) 2012-2014 Mark A. Jensen. This program is free software; you
+Copyright (c) 2012-2020 Mark A. Jensen. This program is free software; you
 can redistribute it and/or modify it under the same terms as Perl
 itself.
 
