@@ -1,10 +1,10 @@
-#$Id: Exceptions.pm 415 2014-05-05 03:00:37Z maj $
+#$Id$
 package REST::Neo4p::Exceptions;
 use strict;
 use warnings;
 
 BEGIN {
-  $REST::Neo4p::Exceptions::VERSION = '0.2250';
+  $REST::Neo4p::Exceptions::VERSION = '0.4000';
 }
 use Exception::Class (
   'REST::Neo4p::Exception',
@@ -25,6 +25,12 @@ use Exception::Class (
     fields => [ 'code' ],
     description => 'Network or HTTP errors'
    },
+  'REST::Neo4p::AuthException' =>
+    {
+    isa => 'REST::Neo4p::Exception',
+    fields => [ 'code', 'message' ],
+    description => 'Authorization error'
+   },
   'REST::Neo4p::NotFoundException' => {
     isa => 'REST::Neo4p::Neo4jException',
     fields => [ 'code', 'neo4j_message', 
@@ -50,9 +56,21 @@ use Exception::Class (
 		  'neo4j_exception', 'neo4j_stacktrace' ],
       description => 'Cypher query language syntax error'
      },
+  'REST::Neo4p::SchemaConstraintExistsException' =>
+    {
+      isa => 'REST::Neo4p::Neo4jException',
+      fields => [qw/error_list neo4j_message code/],
+      description => 'Attempt to create a schema constraint that already exists'
+     },
+  'REST::Neo4p::IndexExistsException' =>
+    {
+      isa => 'REST::Neo4p::Neo4jException',
+      fields => [qw/error_list neo4j_message code/],
+      description => 'Attempt to create an index that already exists'
+     },
   'REST::Neo4p::NotImplException' => {
     isa => 'REST::Neo4p::LocalException',
-    description => 'Attempt to call an currently unimplemented method'
+    description => 'Attempt to call a currently unimplemented method'
    },
   'REST::Neo4p::NotSuppException' => {
     isa => 'REST::Neo4p::LocalException',
@@ -84,6 +102,11 @@ use Exception::Class (
     isa => 'REST::Neo4p::LocalException',
     description => 'The server response body was empty; connection problem?'
    },
+  'REST::Neo4p::StreamException' => {
+    isa => 'REST::Neo4p::LocalException',
+    description => 'Neo4j JSON response parsing error',
+    fields => ['message']
+   },
   'REST::Neo4p::ConstraintException' => {
     isa => 'REST::Neo4p::LocalException',
     description => 'Application-level database constraint violated',
@@ -92,7 +115,13 @@ use Exception::Class (
   'REST::Neo4p::ConstraintSpecException' => {
     isa => 'REST::Neo4p::LocalException',
     description => 'Constraint specification syntax incorrect',
-  }
+  },
+  'REST::Neo4p::Neo4jTightwadException' => {
+    isa => 'REST::Neo4p::LocalException',
+    fields => ['code','message'],
+    description => 'This feature only works in enterprise edition'
+  },
+  
    );
 
 =head1 NAME
@@ -112,7 +141,7 @@ REST::Neo4p::Exceptions - Exception::Class objects for REST::Neo4p
      $RETRIES--;
    };
  } while ( $e = Exception::Class->caught('REST::Neo4p::CommException') );
- ref $e ? $e->rethrow : die $e if $e;
+ (ref $e && $e->can("rethrow")) ? $e->rethrow : die $e if $e;
 
 =head1 Classes
 
@@ -197,7 +226,7 @@ L<REST::Neo4p>, L<Exception::Class>
 
 =head1 LICENSE
 
-Copyright (c) 2012-2014 Mark A. Jensen. This program is free software; you
+Copyright (c) 2012-2020 Mark A. Jensen. This program is free software; you
 can redistribute it and/or modify it under the same terms as Perl
 itself.
 
