@@ -1,4 +1,4 @@
-#$Id: Relationship.pm 415 2014-05-05 03:00:37Z maj $
+#$Id$
 package REST::Neo4p::Relationship;
 use base 'REST::Neo4p::Entity';
 use REST::Neo4p::Exceptions;
@@ -6,14 +6,14 @@ use Carp qw(croak carp);
 use strict;
 use warnings;
 BEGIN {
-  $REST::Neo4p::Relationship::VERSION = '0.2250';
+  $REST::Neo4p::Relationship::VERSION = '0.4000';
 }
 
 sub new {
   my $self = shift;
   my ($from_node, $to_node, $type, $rel_props) = @_;
-  unless (ref $from_node && $from_node->is_a('REST::Neo4p::Node') &&
-	  ref $to_node && $to_node->is_a('REST::Neo4p::Node') &&
+  unless (ref $from_node && $from_node->isa('REST::Neo4p::Node') &&
+	  ref $to_node && $to_node->isa('REST::Neo4p::Node') &&
 	  defined $type) {
     REST::Neo4p::LocalException->throw("Requires 2 REST::Neo4p::Node objects and a relationship type\n");
   }
@@ -37,7 +37,7 @@ sub as_simple {
   my $self = shift;
   my $ret;
   my $props = $self->get_properties;
-  $ret->{_relationship} = $$self;
+  $ret->{_relationship} = $$self + 0;
   $ret->{_type} = $self->type;
   $ret->{_start} = ${$self->start_node};
   $ret->{_end} = ${$self->end_node};
@@ -49,15 +49,31 @@ sub simple_from_json_response {
   my $class = shift;
   my ($decoded_resp) = @_;
   my $ret;
-  # reln id
-  ($ret->{_relationship}) = $decoded_resp->{self} =~ m{.*/([0-9]+)$};
-  # reln type
-  $ret->{_type} = $decoded_resp->{type};
-  # reln properties
-  $ret->{$_} = $decoded_resp->{data}->{$_} for keys %{$decoded_resp->{data}};
-  # start and end nodes (by id)
-  ($ret->{_start}) = $decoded_resp->{start} =~ m{.*/([0-9]+)$};
-  ($ret->{_end}) = $decoded_resp->{end} =~ m{.*/([0-9]+)$};
+  for (ref $decoded_resp) {
+    /HASH/ && do {
+      # reln id
+      ($ret->{_relationship}) = $decoded_resp->{self} =~ m{.*/([0-9]+)$};
+      # reln type
+      $ret->{_type} = $decoded_resp->{type};
+      # reln properties
+      $ret->{$_} = $decoded_resp->{data}->{$_} for keys %{$decoded_resp->{data}};
+      # start and end nodes (by id)
+      ($ret->{_start}) = $decoded_resp->{start} =~ m{.*/([0-9]+)$};
+      ($ret->{_end}) = $decoded_resp->{end} =~ m{.*/([0-9]+)$};
+      last;
+    };
+    /Driver/ && do {
+      $ret->{_relationship} = $decoded_resp->id;
+      $ret->{_type} = $decoded_resp->type;
+      $ret->{$_} = $decoded_resp->properties->{$_} for keys %{$decoded_resp->properties};
+      $ret->{_start} = $decoded_resp->start_id;
+      $ret->{_end} = $decoded_resp->end_id;
+      last;
+    };
+    do {
+      die "?";
+    };
+  }
   return $ret;
 }
 
@@ -67,7 +83,7 @@ REST::Neo4p::Relationship - Neo4j relationship object
 
 =head1 SYNOPSIS
 
- $n1 = REST::Neo4p::Node->new( {name => 'Harry'} )
+ $n1 = REST::Neo4p::Node->new( {name => 'Harry'} );
  $n2 = REST::Neo4p::Node->new( {name => 'Sally'} );
  $r1 = $n1->relate_to($n2, 'met');
  $r1->set_property({ when => 'July' });
@@ -91,6 +107,12 @@ the first argument and second argument, both C<REST::Neo4p::Node>
 objects. An optional fourth argument is a hashref of I<relationship> 
 properties.
 
+=item remove()
+
+ $reln->remove()
+
+Removes the relationship from the database.
+
 =item get_property()
 
  $status = $reln->get_property('status');
@@ -160,7 +182,7 @@ L<REST::Neo4p>, L<REST::Neo4p::Node>, L<REST::Neo4p::Index>.
 
 =head1 LICENSE
 
-Copyright (c) 2012-2014 Mark A. Jensen. This program is free software; you
+Copyright (c) 2012-2020 Mark A. Jensen. This program is free software; you
 can redistribute it and/or modify it under the same terms as Perl
 itself.
 
