@@ -1,4 +1,4 @@
-#$Id: Schema.pm 427 2014-05-13 19:47:26Z maj $
+#$Id$
 use v5.10;
 package REST::Neo4p::Schema;
 use REST::Neo4p::Exceptions;
@@ -7,7 +7,7 @@ use strict;
 use warnings;
 
 BEGIN {
-  $REST::Neo4p::Schema::VERSION = '0.2251';
+  $REST::Neo4p::Schema::VERSION = '0.4000';
 }
 
 #require 'REST::Neo4p';
@@ -41,8 +41,11 @@ sub create_index {
     if (my $e = REST::Neo4p::ConflictException->caught) {
       1; # ignore, already present
     }
+    elsif ( $e = REST::Neo4p::IndexExistsException->caught ) {
+      1;
+    }
     elsif ($e = Exception::Class->caught()) {
-      ref $e ? $e->rethrow : die $e;
+      (ref $e && $e->can("rethrow")) ? $e->rethrow : die $e;
     }
   }
   return 1;
@@ -53,17 +56,19 @@ sub get_indexes {
   my $self = shift;
   my ($label) = @_;
   REST::Neo4p::LocalException->throw("Arg 1 must be a label\n") unless defined $label;
+  my $decoded_resp;
   eval {
-    $self->_agent->get_data(qw/schema index/, $label);
+    $decoded_resp = $self->_agent->get_data(qw/schema index/, $label);
   };
   if (my $e = REST::Neo4p::NotFoundException->caught) {
     return;
   }
   elsif ($e = Exception::Class->caught()) {
-    ref $e ? $e->rethrow : die $e;
+    (ref $e && $e->can("rethrow")) ? $e->rethrow : die $e;
   }
   my @ret;
-  foreach (@{$self->_agent->decoded_content}) {
+  # kludge for Neo4j::Driver
+  foreach (@{$self->_agent->decoded_content // $decoded_resp}) {
     push @ret, $_->{property_keys}[0];
   }
   return @ret;
@@ -81,7 +86,7 @@ sub drop_index {
       1; #ignore if not found
     }
     elsif ($e = Exception::Class->caught()) {
-      ref $e ? $e->rethrow : die $e;
+      (ref $e && $e->can("rethrow")) ? $e->rethrow : die $e;
     }
   }
   return 1;
@@ -111,7 +116,7 @@ sub create_constraint {
       1; # ignore, already present
     }
     elsif ($e = Exception::Class->caught()) {
-      ref $e ? $e->rethrow : die $e;
+      (ref $e && $e->can("rethrow")) ? $e->rethrow : die $e;
     }
   }
   return 1;
@@ -122,17 +127,19 @@ sub get_constraints {
   my ($label, $c_type) = @_;
   $c_type ||= 'uniqueness';
   REST::Neo4p::LocalException->throw("Arg 1 must be a label\n") unless defined $label;
+  my $decoded_resp;
   eval {
-    $self->_agent->get_data(qw/schema constraint/, $label, $c_type);
+    $decoded_resp = $self->_agent->get_data(qw/schema constraint/, $label, $c_type);
   };
   if (my $e = REST::Neo4p::NotFoundException->caught) {
     return;
   }
   elsif ($e = Exception::Class->caught()) {
-    ref $e ? $e->rethrow : die $e;
+    (ref $e && $e->can("rethrow")) ? $e->rethrow : die $e;
   }
   my @ret;
-  foreach (@{$self->_agent->decoded_content}) {
+  # kludge for Neo4j::Driver
+  foreach (@{$self->_agent->decoded_content // $decoded_resp}) {
     push @ret, $_->{property_keys}[0];
   }
   return @ret;
@@ -158,7 +165,7 @@ sub drop_constraint {
       1; # ignore, not initially present
     }
     elsif ($e = Exception::Class->caught()) {
-      ref $e ? $e->rethrow : die $e;
+      (ref $e && $e->can("rethrow")) ? $e->rethrow : die $e;
     }
   }
   return 1;
@@ -177,19 +184,12 @@ REST::Neo4p::Schema - Label-based indexes and constraints
 
 =head1 DESCRIPTION
 
-L<Neo4j|http://neo4j.org> v2.0 provides a way to schematize the graph
+L<Neo4j|http://neo4j.org> v2.0+ provides a way to schematize the graph
 on the basis of node labels, associated indexes, and property
 uniqueness constraints. C<REST::Neo4p::Schema> allows access to this
 system via the Neo4j REST API. Use a C<Schema> object to create, list,
 and drop indexes and constraints.
 
-Note that as of v2.0.0, the Neo4j server can only create indexes on
-single properties within a label, and only uniqueness constraints on
-single properties. v2.0.1 is required for this module, which is
-dependent on a bug fix in that release.
-
-
-
 =head1 METHODS
 
 =over
@@ -224,6 +224,11 @@ Remove indexes on given property or properties for a given label.
 Create uniqueness constraints on a given property or properties for a
 given label.
 
+I<Note>: For some inexplicable reason, this one schema feature went behind
+the paywall in Neo4j version 4.0. Unless you are using the Enterprise
+Edition, this method will throw the dreaded
+L<REST::Neo4p::Neo4jTightwadException>.
+
 =item get_constraints()
 
  @properties = $schema->get_constraints('Label');
@@ -253,7 +258,7 @@ L<REST::Neo4p>, L<REST::Neo4p::Index>, L<REST::Neo4p::Query>
 
 =head1 LICENSE
 
-Copyright (c) 2012-2014 Mark A. Jensen. This program is free software; you
+Copyright (c) 2012-2020 Mark A. Jensen. This program is free software; you
 can redistribute it and/or modify it under the same terms as Perl
 itself.
 
